unit ifwic;

{$IMPORTEDDATA OFF}
{$MINENUMSIZE 4}

interface

uses ImgFilter;

type
  TWicFilter = class(TImgFilter)
    constructor Create; override;
    function Test(Buf: PChar; BufSize: Cardinal): Boolean; override;
    procedure Decode(InBuf: PChar; InBufSize: Cardinal;
        var OutBuf: PChar; var OutBufSize: Cardinal; var ImgInfo: TImgInfo);
        override;
  end;

implementation

uses Windows, ActiveX, WincodecSub, IcmSub, NLib, NSys, IccpUtil;

var
  HasWebpDecoder: (hdUnknown, hdTrue, hdFalse) = hdUnknown;

procedure WicCheck(Value: HRESULT);
begin
  if Value <> 0 then
    raise EImgFilterError.Create('WIC error: ' + SysError(Value));
end;

constructor TWicFilter.Create;
var
  factory: IWICImagingFactory;
  decoder: IWICBitmapDecoder;
  r: HRESULT;
begin
  FName := 'Internal WIC WebP decoder';
  if HasWebpDecoder <> hdUnknown then
    Exit;
  r := CoCreateInstance(CLSID_WICImagingFactory, nil, CLSCTX_INPROC_SERVER,
      IWICImagingFactory, factory);
  if (r = S_OK) and (factory.CreateDecoder(GUID_ContainerFormatWebp, nil, decoder) = 0) then
    HasWebpDecoder := hdTrue
  else
    HasWebpDecoder := hdFalse;
end;

function TWicFilter.Test(Buf: PChar; BufSize: Cardinal): Boolean;
  function match(p: PChar; ofs, len: Cardinal): Boolean;
  begin
    Result := ((ofs + len) <= BufSize) and (MemCmp(@Buf[ofs], p, len) = 0);
  end;
begin
  Result := (HasWebpDecoder = hdTrue) and match('RIFF', 0, 4) and match('WEBP', 8, 4);
end;

procedure TWicFilter.Decode(InBuf: PChar; InBufSize: Cardinal;
    var OutBuf: PChar; var OutBufSize: Cardinal; var ImgInfo: TImgInfo);
const
  LCS_WINDOWS_COLOR_SPACE = $57696E20;
var
  buf, ps, pd: PChar;
  bufsize, stride, a, x, y, f: Cardinal;
  st: IWICStream;
  factory: IWICImagingFactory;
  decoder: IWICBitmapDecoder;
  decode: IWICBitmapFrameDecode;
  pf: TGUID;
  ccsrc, ccdst: IWICColorContext;
  dstpath: WideString;
  dstpathsize: DWORD;
  ct: IWICColorTransform;
  cctype: WICColorContextType;
  cp: PByte;
  w, h, ccscount, cpsize: UINT;
begin
  WicCheck(CoCreateInstance(CLSID_WICImagingFactory, nil, CLSCTX_INPROC_SERVER,
      IWICImagingFactory, factory));
  WicCheck(factory.CreateDecoder(GUID_ContainerFormatWebp, nil, decoder));
  WicCheck(factory.CreateStream(st));
  WicCheck(st.InitializeFromMemory(InBuf, InBufSize));
  WicCheck(decoder.Initialize(st, 0));
  WicCheck(decoder.GetFrame(0, decode));
  WicCheck(decode.GetPixelFormat(pf));
  if not IsEqualGUID(pf, GUID_WICPixelFormat32bppRGBA) then
    raise EImgFilterError.Create('unsupported pixel format');

  WicCheck(factory.CreateColorContext(ccsrc));
  WicCheck(decode.GetColorContexts(1, ccsrc, ccscount));
  WicCheck(factory.CreateColorContext(ccdst));
  GetStandardColorSpaceProfileW(nil, LCS_WINDOWS_COLOR_SPACE, nil, @dstpathsize);
  SetLength(dstpath, dstpathsize div 2 - 1);
  if not GetStandardColorSpaceProfileW(nil, LCS_WINDOWS_COLOR_SPACE,
      PWideChar(dstpath), @dstpathsize) then
    raise EOSError.Create;
  WicCheck(ccdst.InitializeFromFilename(PWideChar(dstpath)));
  WicCheck(factory.CreateColorTransformer(ct));
  WicCheck(ct.Initialize(decode, ccsrc, ccdst, pf));

  WicCheck(ct.GetSize(w, h));

  OutBufSize := ((w * 3 + 3) and not 3) * h;
  OutBuf := Alloc(OutBufSize);
  if OutBuf = nil then
    raise EImgFilterError.Create('output buffer allocation failed');

  stride := w * 4;
  bufsize := stride * h;
  GetMem(buf, bufsize);
  WicCheck(ct.CopyPixels(nil, stride, bufsize, buf));

  ps := buf + bufsize - w * 4;
  pd := OutBuf;
  y := 0;
  while y < h do begin
    f := (h-1 - y) and 8;
    x := 0;
    while x < w do begin
      // blend 8x8 checkered pattern
      // (255 - alpha) * (160 <or> 128) shr 8
      a := (Cardinal(255) - Ord(ps[3])) * (160 - (x and 8 xor f) shl 2) shr 8;
      pd[0] := Char((Ord(ps[2]) + 1) * Ord(ps[3]) shr 8 + a);
      pd[1] := Char((Ord(ps[1]) + 1) * Ord(ps[3]) shr 8 + a);
      pd[2] := Char((Ord(ps[0]) + 1) * Ord(ps[3]) shr 8 + a);
      Inc(ps, 4);
      Inc(pd, 3);
      Inc(x);
    end;
    Inc(pd, w and 3);
    Dec(ps, stride * 2);
    Inc(y);
  end;

  WicCheck(ccsrc.GetType(cctype));
  if cctype = WICColorContextProfile then begin
    WicCheck(ccsrc.GetProfileBytes(0, nil, cpsize));
    GetMem(cp, cpsize);
    try
      WicCheck(ccsrc.GetProfileBytes(cpsize, cp, cpsize));
      ImgInfo.ICCProfileName := GetICCProfileDesc(PChar(cp), cpsize);
    finally
      FreeMem(cp);
    end;
  end;

  FreeMem(buf);

  ImgInfo.Width := w;
  ImgInfo.Height := h;
  ImgInfo.Components := 3;
  ImgInfo.Bpp := 24;
  ImgInfo.FormatName := '';
end;

end.
