unit InfoDialog;

{$IMPORTEDDATA OFF}

interface

uses NForm, NGraphics, NkvImage;

type
  TInfoDialog = class(TDialog)
    Icon: TIcon;
    Path: WideString;
    PathInArchive: WideString;
    ArchivePluginName: string;
    ArchivePluginFileName: WideString;
    Image: TNkVImage;
    MTime: Int64;
    DirStatError: Boolean;
    constructor Create; override;
    destructor Destroy; override;
    procedure Init(var FocusId: Integer); override;
    procedure UpdateInfo;
  end;

implementation

uses NLib, NSys, NDesktop;

{$I dialogid.inc}

function BytesFormat(Value: Int64): string;
var
  p: PChar;
  u: string;
begin
  Result := UnitFormat(Value);
  p := @Result[Length(Result)];
  if p^ in ['K', 'M', 'G'] then begin
    u := p^;
    p^ := ' ';
    Result := Result + u + 'B';
  end else
    Result := Result + ' bytes';
end;

constructor TInfoDialog.Create;
begin
  inherited;
  TemplateName := 'INFODIALOG';
end;

destructor TInfoDialog.Destroy;
begin
  if Assigned(Icon) then
    Icon.Free;
  inherited;
end;

procedure TInfoDialog.Init(var FocusId: Integer);
begin
  inherited;
  UpdateInfo;
end;

procedure TInfoDialog.UpdateInfo;
var
  t: TTime;
  plugininfo: WideString;
begin
  if not Assigned(Icon) then
    Icon := TIcon.Create;
  if Path = '' then begin
    LoadFileIcon(Icon, 'shell32.dll', 2);
    Items[IDC_FILENAME].SetText('<STDIN>');
  end else begin
    LoadFileIcon(Icon, Path);
    if PathInArchive = '' then
      Items[IDC_FILENAME].SetText(BaseName(Path))
    else
      Items[IDC_FILENAME].SetText(PathInArchive)
  end;
  SetIcon(IDC_ICON, Icon);

  if PathInArchive = '' then begin
    if Path = '' then begin
      Items[IDC_LOCATIONLABEL].Enable(False);
      Items[IDC_LOCATION].SetText('');
    end else begin
      Items[IDC_LOCATIONLABEL].Enable(True);
      Items[IDC_LOCATION].SetText(DirName(Path));
    end;
  end else
    Items[IDC_LOCATION].SetText(Path);
  Items[IDC_SIZE].SetText(Format('{}x{}', [Image.Width, Image.Height]));
  if Image.Bpp > 0 then
    Items[IDC_BPP].SetText(Format('{} bits/pixel', [Image.Bpp]))
  else
    Items[IDC_BPP].SetText('? bits/pixel');
  Items[IDC_FORMAT].SetText(Image.FormatName);

  Items[IDC_ICCPROFILELABEL].Enable(Image.ICCProfileSupported);
  Items[IDC_ICCPROFILE].SetText(Image.ICCProfileName);

  Items[IDC_FILESIZE].SetText(Format('{} ({} bytes)',
      [BytesFormat(Image.OrgSize), NumFormat(Image.OrgSize)]));
  if (Path = '') or DirStatError then begin
    Items[IDC_MODIFIEDLABEL].Enable(False);
    Items[IDC_MODIFIED].SetText('');
  end else begin
    Items[IDC_MODIFIEDLABEL].Enable(True);
    t := LocalTime(MTime);
    Items[IDC_MODIFIED].SetText(
        Format('{:04}-{:02}-{:02} {:02}:{:02}:{:02}',
          [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  end;
  Items[IDC_MEMORY].SetText(
      Format('{} ({} bytes)',
        [BytesFormat(Image.BitsSize), NumFormat(Image.BitsSize)]));

  plugininfo := Image.FilterName;
  if Image.FilterFileName <> '' then
    plugininfo := plugininfo + #13#10'  ' + Image.FilterFileName;
  if ArchivePluginName <> '' then
    plugininfo := plugininfo + #13#10 +
        ArchivePluginName + #13#10'  ' + ArchivePluginFileName;
  Items[IDC_PLUGIN].SetText(plugininfo);

  Items[IDC_TEXTINFOLABEL].Enable(Image.Comment <> '');
  Items[IDC_TEXTINFO].SetText(Image.Comment);
end;

end.
