unit ShellUtils;

{$IMPORTEDDATA OFF}

interface

uses Windows, ShlObj;

type
  TContextMenu = class(TObject)
    ContextMenu2: IContextMenu2;
    Menu: HMENU;
    Parent: HWND;
    constructor Create(Parent: HWND; const Path: WideString);
    destructor Destroy; override;
    procedure HandleMenuMsg(var Message);
    procedure Popup(SX, SY: Integer);
  end;

implementation

uses Messages, ActiveX, NSys, NLib;

constructor TContextMenu.Create(Parent: HWND; const Path: WideString);
var
  ma: IMalloc;
  dir, fname: WideString;
  desktop, folder: IShellFolder;
  len, attr: ULONG;
  dpidl, pidl: PItemIdList;
  cm: IContextMenu;
begin
  Self.Parent := Parent;
  Menu := CreatePopupMenu;
  if Menu = 0 then
    raise EOSError.Create;
  dir := DirName(path);
  fname := BaseName(path);
  if CoGetMalloc(1, ma) < 0 then
    raise EOSError.Create;
  if SHGetDesktopFolder(desktop) < 0 then
    raise EOSError.Create;
  try
    dpidl := nil;
    pidl := nil;
    if desktop.ParseDisplayName(0, nil, PWideChar(dir), len, dpidl, attr) < 0 then
      raise EOSError.Create(Path);
    if desktop.BindToObject(dpidl, nil, IShellFolder, folder) < 0 then
      raise EOSError.Create(Path);
    if folder.ParseDisplayName(0, nil, PWideChar(fname), len, pidl, attr) < 0 then
      raise EOSError.Create(Path);
    if folder.GetUIObjectOf(Parent, 1, pidl, IContextMenu, nil, cm) < 0 then
      raise EOSError.Create(Path);
    ContextMenu2 := cm as IContextMenu2;
    if ContextMenu2.QueryContextMenu(Menu, 0, 1, $7fff, CMF_EXPLORE) < 0 then
      raise EOSError.Create(Path);
  finally
    ma.Free(dpidl);
    ma.Free(pidl);
  end;
end;

destructor TContextMenu.Destroy;
begin
  ContextMenu2 := nil;
  DestroyMenu(Menu);
end;

procedure TContextMenu.Popup(SX, SY: Integer);
var
  id: Integer;
  info: TCMInvokeCommandInfo;
begin
  id := Integer(TrackPopupMenu(Menu,
      TPM_LEFTALIGN or TPM_RIGHTBUTTON or TPM_RETURNCMD, SX, SY, 0, Parent,
      nil));
  if Id <> 0 then
  begin
    FillChar(info, SizeOf(Info), 0);
    info.cbSize := SizeOf(info);
    info.lpVerb := MAKEINTRESOURCE(id-1);
    info.nShow := SW_SHOWNORMAL;
    info.hWnd := Parent;
    ContextMenu2.InvokeCommand(info);
  end;
end;

procedure TContextMenu.HandleMenuMsg(var Message);
begin
  if Assigned(ContextMenu2) then
    with TMessage(Message) do
      case Msg Of
        WM_INITMENUPOPUP, WM_DRAWITEM, WM_MEASUREITEM:
          ContextMenu2.HandleMenuMsg(Msg, wParam, lParam);
      end;
end;

end.
